import cherrypy
from cherrytemplate import renderTemplate

from utils import *
import shutil, os, cStringIO, gzip, bz2, tarfile
from datetime import datetime

from connection import conn
from sqlobject import *

from index import WebRoot
from news import News


class Document(SQLObject):
    _connection = conn
    _defaultOrder=["name", "-revision"]
    name = StringCol()
    description = StringCol(default="")
    author = StringCol()
    revision = StringCol()
    html = BoolCol(default=False)
    src = StringCol(default="")
    pdf = StringCol(default="")


class WebDocument(WebRoot):
    def index(self):
        return self.list()

    index.exposed = True

    def default(self, *args, **kwargs):
        if hasattr(self, args[0]):
            return getattr(self, args[0])(*args[1:], **kwargs)
    default.exposed = True


    def view(self, did, *path):
        basepath = os.path.join("docs", str(did), "html")
        index = "index"

        if os.path.exists(os.path.join(basepath, index + ".htm")):
            index += ".htm"
        elif os.path.exists(os.path.join(basepath, index + ".html")):
            index += ".html"
        elif os.path.exists(os.path.join(basepath, index + ".xml")):
            index += ".xml"
        # Get the first file
        else:
            files = os.listdir(basepath)
            for f in files:
                if os.path.isfile(os.path.join(basepath, f)):
                    index = f
                    break

        if len(path) == 0:
            f = os.path.join(basepath, index)
        else:
            f = basepath
            for p in path:
                f += "/" + p

        html = "<div class='documents_iframe'> \
            <a href='/" + f + "' target='_blank'>View in new window</a> \
            <iframe src ='/" + f  + "'></iframe> \
          </div>"
        
        return self.render(html)
    
    view.exposed = True

    def _upload(self, did,  document, html=False):
        # document path
        basepath = os.path.join("docs", str(did))
        if not os.path.exists(basepath):
            os.mkdir(basepath)

        # html case
        if html:
            # html directory path
            htmlpath = os.path.join(basepath, "html")
            if not os.path.exists(htmlpath):
                os.mkdir(htmlpath)

            ext1 = document.filename.split(".")[-1]
            ext2 = document.filename.split(".")[-2]

            # Not compressed
            if ext1 == "html" or ext1 == "htm" or ext1 == "xml":
                data = self._read_data(document.file)
                self._write_file(htmlpath, "index."+ext1, data)
                
            # Compressed
            elif ext1 == "gz" or ext1 == "bz2":
                try:
                    # Gzip
                    if ext1 == "gz":
                        zfile = gzip.GzipFile(mode='rb', fileobj=document.file)
                        data = self._read_data(zfile)
                    
                    # Bzip2
                    else:
                        zfile = self._read_data(document.file)
                        data = bz2.decompress(zfile)

                    # Tarball
                    if ext2 == "tar":
                        self._decompress_tar(basepath, data)
                    else:
                        self._write_file(htmlpath, "index."+ext2, data)
                except Exception:
                    return False
    
            # Extention not supported
            else:
                return False

            return True
        
        else:
            data = self._read_data(document.file)
            self._write_file(basepath, document.filename, data)

            
    def _read_data(self, fileobj):
        data = fileobj.read(1024 * 8)
        while 1:
            tmp = fileobj.read(1024 * 8)
            if not tmp: break
            data += tmp
        
        return data
    
    def _write_file(self, path, filename, data):
        dstname = os.path.join(path, filename)

        # write file
        f = open(dstname, 'wb')
        f.write(data)
        f.close()

    def _decompress_tar(self, path, data):
        # paths
        htmlpath = os.path.join(path, "html")
        tmppath = os.path.join(path, "tmp")
        if not os.path.exists(tmppath):
            os.mkdir(tmppath)
                        
        # extract files
        tardata = cStringIO.StringIO(data)
        tar = tarfile.open(mode='r:', fileobj=tardata)

        members = tar.getnames()
        for el in members:
            tar.extract(el, tmppath)
            
        # list files
        files = os.listdir(tmppath)
        nb = len(files)

        # move to html directory
        if nb == 1:
            dirpath = os.path.join(tmppath, files[0])
            shutil.move(dirpath, htmlpath)
            os.rmdir(tmppath)
        else:
            shutil.move(tmppath, htmlpath)

        tar.close()

                    
    def list(self):
        documents = Document.select()
        html = renderTemplate(file="templates/documents_list.html")
        return self.render(html)

    list.exposed = True


    def add(self, name=None, description=None, author=None, revision=None, html=None, src=None, pdf=None):
        if name and author and revision:
            document = Document(name=name,
                                author=author,
                                revision=revision)

            bad_file = False
            
            if description:
                document.description = description
            
            if src.filename != "":
                self._upload(document.id, src)
                document.src = src.filename
                
            if pdf.filename != "":
                pdfext = pdf.filename.split(".")[-1]

                if pdfext == "ps" or pdfext == "pdf":
                    self._upload(document.id, pdf)
                    document.pdf = pdf.filename
                else:
                    bad_file = True
                    
            if html.filename != "":
                if self._upload(document.id, html, True):
                    document.html = True
                else:
                    bad_file = True

            if bad_file:
                return self.render_page("document_upload_warning")
            
            raise cherrypy.HTTPRedirect("/documents")
        else:
            return self.render_page("admin_document_add")

    add = adminOnly(add)

    def edit(self, did, name=None, description=None, author=None, revision=None, html=None, src=None, pdf=None):
        document = Document.get(did)

        if name and author and revision:
            document.name = name
            document.author = author
            document.revision = revision

            bad_file = False
            
            if description:
                document.description = description

            if src.filename != "":
                self.del_data(did, "src")
                self._upload(did, src)
                document.src = src.filename

            if pdf.filename != "":
                pdfext = pdf.filename.split(".")[-1]

                if pdfext == "ps" or pdfext == "pdf":
                    self.del_data(did, "pdf")
                    self._upload(document.id, pdf)
                    document.pdf = pdf.filename
                else:
                    bad_file = True

            if html.filename != "":
                self.del_data(did, "html")
                if self._upload(document.id, html, True):
                    document.html = True
                else:
                    bad_file = True
            
            if bad_file:
                return self.render_page("document_upload_warning")

            raise cherrypy.HTTPRedirect("/documents")
        else:
            html = renderTemplate(file="templates/admin_document_edit.html")
            return self.render(html)
        
    edit = adminOnly(edit)

    def delete(self, did, confirmed=None):
        document = Document.get(did)

        if confirmed:
            docpath = os.path.join("docs", did)
            if os.path.exists(docpath):
                remove_dir(docpath)
            
            Document.delete(did)
            
            raise cherrypy.HTTPRedirect("/documents")
        else:
            conf_msg="Do you really want to delete document %s - %s ?" % (document.name, document.revision)
            conf_yes="delete"
            conf_hidden = { "did" : did,
                            "confirmed" : "yes" }
            
            html = renderTemplate(file="templates/confirm_action.html")
            
            return self.render(html)
        
    delete = adminOnly(delete)

    def del_data(self, did, data, ref=None):
        document = Document.get(did)

        if data == "html":
            htmlpath = os.path.join("docs", did)
            if os.path.exists(htmlpath):
                remove_dir(htmlpath)

            document.html = False
        
        elif data == "src":
            dstname = os.path.join("docs", did, document.src)
            if os.path.exists(dstname) and os.path.isfile(dstname):
                os.unlink(dstname)

            document.src = ""

        elif data == "pdf":
            dstname = os.path.join("docs", did, document.pdf)
            if os.path.exists(dstname) and os.path.isfile(dstname):
                os.unlink(dstname)

            document.pdf = ""

        if ref:
            raise cherrypy.HTTPRedirect(ref)

    del_data = adminOnly(del_data)
    



